@@ -1,7 +1,43 @@
 #' @include HelperFunctions.R
 NULL
 
-#' Initialize a data structure for the definition of [InsuranceTarif] costs
+#' Update one component of an [InsuranceTarif]'s cost structure
+#'
+#' Insurance tariff costs are defined by a cost matrix with dimensions: {CostType, Basis, Period}, where:
+#' \describe{
+#'     \item{CostType:}{alpha, Zillmer, beta, gamma, gamma_nopremiums, unitcosts}
+#'     \item{Basis:}{SumInsured, SumPremiums, GrossPremium, NetPremium, Constant}
+#'     \item{Period:}{once, PremiumPeriod, PremiumFree, PolicyPeriod, AfterDeath, FullContract}
+#' }
+#' After creation (using the function [initializeCosts()]), the funciton \code{setCost}
+#' can be used to modify a single entry. While [initializeCosts()] provides arguments
+#' to set the most common types of cost parameters in one call, the \code{setCost}
+#' function allows to modify any cost parameter.
+#'
+#' This function modifies a copy of the cost structure and returns it, so this
+#' function can be chained as often as desired and final return value will contain
+#' all cost modifications.
+#'
+#' @param costs The cost definition matrix (usually created by [initializeCosts()])
+#' @param type The cost type (alpha, Zillmer, beta, gamma, gamma_nopremiums, unitcosts)
+#' @param basis The basis fo which the cost rate is applied (default is SumInsured)
+#' @param frequency How often / during which period the cost is charged (once, PremiumPeriod, PremiumFree, PolicyPeriod, FullContract)
+#' @param value The new cost value to set for the given type, basis and frequency
+#'
+#' @return The modified cost structure
+#'
+#' @examples
+#' costs = initializeCosts()
+#' setCost(costs, "alpha", "SumPremiums", "once", 0.05)
+#'
+#' @export
+setCost = function(costs, type, basis = "SumInsured", frequency = "PolicyPeriod", value) {
+  costs[type, basis, frequency] = value
+  costs
+}
+
+
+#' Initialize or modify a data structure for the definition of [InsuranceTarif] costs
 #'
 #' Initialize a cost matrix with dimensions: {CostType, Basis, Period}, where:
 #' \describe{
@@ -9,7 +45,7 @@ NULL
 #'     \item{Basis:}{SumInsured, SumPremiums, GrossPremium, NetPremium, Constant}
 #'     \item{Period:}{once, PremiumPeriod, PremiumFree, PolicyPeriod}
 #' }
-#' This cost structure can then be modified for non-standard costs.
+#' This cost structure can then be modified for non-standard costs using the [setCost()] function.
 #' The main purpose of this structure is to be passed to [InsuranceContract] or
 #' [InsuranceTarif] definitions.
 # TODO: gamma an Erlebensleistungen?
@@ -22,6 +58,9 @@ NULL
 #' @param gamma.paidUp Administration costs for paid-up contracts (relative to sum insured)
 #' @param gamma.premiumfree Administration costs for planned premium-free period (reltaive to sum insured)
 #' @param gamma.contract Administration costs for the whole contract period (relative to sum insured)
+#' @param gamma.afterdeath Administration costs after the insured person has dies (for term-fix insurances)
+#' @param gamma.fullcontract Administration costs for the full contract period,
+#'                           even if the insured has already dies (for term-fix insurances)
 #' @param unitcosts Unit costs (absolute monetary amount, during premium period)
 #' @param unitcosts.PolicyPeriod Unit costs (absolute monetary amount, during full contract period)
 #'
@@ -39,15 +78,24 @@ NULL
 #' costs.Bsp[["beta", "GrossPremium", "PremiumPeriod"]] = 0.05;
 #' costs.Bsp[["gamma", "SumInsured", "PolicyPeriod"]] = 0.001;
 #'
+#' # The same structure using the setCost() function:
+#' library(magrittr)
+#' costs.Bsp = initializeCosts() %>%
+#'   setCost("alpha", "SumPremiums", "once", 0.04) %>%
+#'   setCost("Zillmer", "SumPremiums", "once", 0.025) %>%
+#'   setCost("beta", "GrossPremium", "PremiumPeriod", 0.05) %>%
+#'   setCost("gamma", "SumInsured", "PolicyPeriod", 0.001)
+#'
+#'
 #'
 #'
 #' @export
-initializeCosts = function(costs, alpha, Zillmer, beta, gamma, gamma.paidUp, gamma.premiumfree, gamma.contract, unitcosts, unitcosts.PolicyPeriod) {
+initializeCosts = function(costs, alpha, Zillmer, beta, gamma, gamma.paidUp, gamma.premiumfree, gamma.contract, gamma.afterdeath, gamma.fullcontract, unitcosts, unitcosts.PolicyPeriod) {
     if (missing(costs)) {
         dimnm = list(
             type = c("alpha", "Zillmer", "beta", "gamma", "gamma_nopremiums", "unitcosts"),
             basis = c("SumInsured", "SumPremiums", "GrossPremium", "NetPremium", "Constant", "Reserve"),
-            frequency = c("once", "PremiumPeriod", "PremiumFree", "PolicyPeriod")
+            frequency = c("once", "PremiumPeriod", "PremiumFree", "PolicyPeriod", "AfterDeath", "FullContract")
         );
         costs = array(
             0,
@@ -56,35 +104,80 @@ initializeCosts = function(costs, alpha, Zillmer, beta, gamma, gamma.paidUp, gam
         );
     }
     if (!missing(alpha)) {
-        costs[["alpha",  "SumPremiums", "once"]] = alpha;
+      costs = setCost(costs, "alpha",  "SumPremiums", "once", alpha)
     }
     if (!missing(Zillmer)) {
-        costs[["Zillmer","SumPremiums", "once"]] = Zillmer;
+      costs = setCost(costs, "Zillmer","SumPremiums", "once", Zillmer)
     }
     if (!missing(beta))  {
-        costs[["beta", "GrossPremium", "PremiumPeriod"]] = beta;
+      costs = setCost(costs, "beta", "GrossPremium", "PremiumPeriod", beta)
     }
     if (!missing(gamma)) {
-        costs[["gamma", "SumInsured", "PremiumPeriod"]] = gamma;
+      costs = setCost(costs, "gamma", "SumInsured", "PremiumPeriod", gamma)
     }
     if (!missing(gamma.premiumfree)) {
-        costs[["gamma", "SumInsured", "PremiumFree"]] = gamma.premiumfree;
+      costs = setCost(costs, "gamma", "SumInsured", "PremiumFree", gamma.premiumfree)
     }
     if (!missing(gamma.paidUp))  {
-        costs[["gamma_nopremiums", "SumInsured", "PolicyPeriod"]] = gamma.paidUp;
+      costs = setCost(costs, "gamma_nopremiums", "SumInsured", "PolicyPeriod", gamma.paidUp)
     }
     if (!missing(gamma.contract))  {
-        costs[["gamma", "SumInsured", "PolicyPeriod"]] = gamma.contract;
+      costs = setCost(costs, "gamma", "SumInsured", "PolicyPeriod", gamma.contract)
+      # costs = setCost(costs, "gamma_nopremiums", "SumInsured", "PolicyPeriod", gamma.contract)
+    }
+    if (!missing(gamma.afterdeath))  {
+      costs = setCost(costs, "gamma", "SumInsured", "AfterDeath", gamma.afterdeath)
+      # costs = setCost(costs, "gamma_nopremiums", "SumInsured", "AfterDeath", gamma.afterdeath)
+    }
+    if (!missing(gamma.fullcontract))  {
+      costs = setCost(costs, "gamma", "SumInsured", "FullContract", gamma.fullcontract)
+      # costs = setCost(costs, "gamma_nopremiums", "SumInsured", "FullContract", gamma.fullcontract)
     }
     if (!missing(unitcosts)) {
-        costs[["unitcosts", "Constant", "PremiumPeriod"]] = unitcosts;
+      costs = setCost(costs, "unitcosts", "Constant", "PremiumPeriod", unitcosts)
     }
     if (!missing(unitcosts.PolicyPeriod)) {
-        costs[["unitcosts", "Constant", "PolicyPeriod"]] = unitcosts.PolicyPeriod;
+      costs = setCost(costs, "unitcosts", "Constant", "PolicyPeriod", unitcosts.PolicyPeriod)
     }
     costs
 }
 
+#' Helper function to define base costs with base alpha, but otherwise unchanged costs
+#'
+#' Returns a function that sets base alpha (and Zillmer) costs to the given value,
+#' but otherwise uses the full costs defined by the Costs parameter.
+#'
+#' This function can be set as minCosts parameter for a tariff and makes sure
+#' that only alpha costs are modified / waived, but no other costs.
+#'
+#' @param alpha The minimum alpha / Zillmer cost that cannot be waived
+#'
+#' @export
+costs.baseAlpha = function(alpha) {
+  function(params, values, costs) {
+    costs = setCost(costs, "alpha", "SumPremiums", "once", alpha)
+    if (costs["Zillmer", "SumPremiums", "once"] != 0) {
+      costs = setCost(costs, "Zillmer", "SumPremiums", "once", alpha)
+    }
+    costs
+  }
+}
+
+
+#' Helper function to display all cost definitions in a concise table
+#'
+#' Returns a data.frame with columns
+#'
+#' @param costs The cost structure to be displayed in a concise table style.
+#' @export
+costsDisplayTable = function(costs) {
+  costtable = as.data.frame.table(setInsuranceValuesLabels(costs) )
+  colnames(costtable) = c("Kostenart", "Basis", "Periode", "Kostensatz");
+  costtable[costtable[,"Kostensatz"] != 0.0000,]
+}
+
+
+
 
 #' Data structure (filled only with NULL) for insurance contract class member values.
 #' @export
@@ -113,7 +206,7 @@ InsuranceContract.Values = list(
     profitParticipation = list()
 );
 
-# InsuranceContract.ParameterDefault #######################################
+# InsuranceContract.ParameterDefaults #######################################
 #' Default parameters for the InsuranceContract class. A new contract will be
 #' pre-filled with these values, and values passed in the constructor (or with
 #' other setter functions) will override these values.
@@ -129,7 +222,7 @@ InsuranceContract.Values = list(
 #'     \item \code{$ActuarialBases} ... Actuarial bases for the contract
 #'               calculation (mortality/invalidity table, guaranteed interest,
 #'               surrender penalty, etc.)
-#'     \item \code{$Costs} ... Expenses charged to the contract (see [initializeCosts()])
+#'     \item \code{$Costs}, \code{$minCosts} ... Expenses charged to the contract (see [initializeCosts()])
 #'     \item \code{$Loadings} ... Loadings, rebates and other charges of the
 #'               tariff / contract (tax, unit costs, surcharge for no medial exam, premium/benefit frequency loading)
 #'     \item \code{$Features} ... Peculiarities of the tariff (to enable
@@ -153,6 +246,9 @@ InsuranceContract.Values = list(
 #'               contracts with multiple parts, e.g. dynamic increases),
 #'               default = "Hauptvertrag"}
 #'     \item{\code{$sumInsured}}{Sum insured, default = 100,000}
+#'     \item{\code{$initialCapital}}{Reserve/Capital that is already available
+#'               at contract inception, e.g. from a previous contract. No tax
+#'               or acquisition costs are applied to this capital.}
 #'     \item{\code{$YOB}}{Year of birth of the insured, used to determine the
 #'               age for the application of the mortality table}
 #'     \item{\code{$age}}{Age of the insured}
@@ -192,7 +288,7 @@ InsuranceContract.Values = list(
 #'               is aligned with $t=blockStart$ of the parent block.}
 #'     \item{\code{$premiumPayments}}{Whether premiums are paid in advance
 #'               (default) or arrears. Value is of type [PaymentTimeEnum]
-#'               with possible values "in advance" and 'in arrears"}
+#'               with possible values "in advance" and "in arrears"}
 #'     \item{\code{$benefitPayments}}{Whether recurring benefits (e.g. annuities)
 #'               are paid in advance (default) or arrears. Value is of type
 #'               [PaymentTimeEnum] with possible values "in advance" and
@@ -216,6 +312,12 @@ InsuranceContract.Values = list(
 #'     \item{\code{$deathBenefit}}{The yearly relative death benefit (relative
 #'               to the initial sum insured); Can be set to a \code{function(len,
 #'               params, values)}, e.g. \code{deathBenefit = deathBenefit.linearDecreasing}}
+#'
+#'    \item{\code{$costWaiver}}{The fraction of the costs that are waived (only
+#'               those cost components that are defined to be waivable, i.e. by
+#'               defining a corresponding \code{$minCosts}). Linearly interpolates
+#'               between \code{$Costs} and \code{$minCosts}, if the latter is set.
+#'               Otherwise is has no effect.}
 #' }
 #'
 #' ## Elements of sublist \code{InsuranceContract.ParameterDefault$ContractState}
@@ -255,9 +357,22 @@ InsuranceContract.Values = list(
 #'               month are relevant.}
 #'     \item{\code{$balanceSheetMethod}}{How to interpolate the balance sheet
 #'               reserves (at the balandeSheetDate) from the yearly contractual
-#'               reserves.}
+#'               reserves. Either a string "30/360", "act/act", "act/360", "act/365"
+#'               or a function with signature \code{balanceSheetMethod(params, contractDates, balanceDates)}
+#'               that returns a vector of coefficients for each year to
+#'               interpolate the reserves available at the given \code{contractDates}
+#'               for the desired \code{balanceDates}}
+#'     \item{\code{$unearnedPremiumsMethod}}{How to calculate the unearned
+#'               premiums (considering the balance sheet date and the premium
+#'               frequency). A function with signature \code{unearnedPremiumsMethod(params, dates)}}
 #'     \item{\code{$surrenderValueCalculation}}{A function describing the surrender
-#'               value calculation.}
+#'               value calculation. If NULL, the full reserve will be used as
+#'               surrender value. If given, it must be a function with signature
+#'               \code{function(SurrenderReserve, params, values)}.}
+#'     \item{\code{$premiumWaiverValueCalculation}}{A function describing the
+#'               reserve used to derive the premium-free sum insured. If NULL,
+#'               the surrender value will be used. If given, it must be a function
+#'               with signature \code{function(SurrenderReserve, params, values)}}
 #'     \item{\code{$premiumFrequencyOrder}}{Order of the approximation for
 #'               payments within the year (unless an extra frequency loading is
 #'               used => then leave this at 0)}
@@ -270,6 +385,17 @@ InsuranceContract.Values = list(
 #'
 #' Definition of contractual costs charged to the contract. See [initializeCosts()].
 #'
+#' \describe{
+#'     \item{\code{$Costs}}{The full cost defined for the contract / tariff,
+#'               usually set with [initializeCosts()] and [setCost()]}
+#'     \item{\code{$minCosts}}{The minimum costs defined for the contract / tariff
+#'               that cannot be waived. Either an explicit cost definition structure
+#'               generated by [initializeCosts()] and [setCost()], or a
+#'               `function(params, values, costs)`, where the full costs are passed
+#'               as third parameter, so the function can modify only those cost parts
+#'               that can be waived at all. }
+#' }
+#'
 #' ## Elements of sublist \code{InsuranceContract.ParameterDefault$Loadings}
 #'
 #' \describe{
@@ -305,6 +431,18 @@ InsuranceContract.Values = list(
 #'               included in the Zillmer premium calculation}
 #'     \item{\code{$alphaRefundLinear}}{Whether the refund of alpha-costs on
 #'               surrender is linear in t or follows the NPV of an annuity}
+#'     \item{\code{$useUnearnedPremiums}}{Whether unearned premiums should be
+#'               reported in the balance sheet reserves. Otherwise, a premium
+#'               paid at the beginning of the period is added to the reserve at
+#'               that time for balance-sheet purposes.
+#'               For regular premiums, the default is TRUE, i.e. the balance-sheet
+#'               reserve at time $t$ does not include the premium paid at time
+#'               $t$, but unearned premiums are included in the balance sheet
+#'               reserves. For single-premium contracts, there are no "unearned"
+#'               premiums, but the initial single premium is added to the reserve
+#'               immediately for balance-sheet purposes. In particular, the
+#'               balance sheet reserve at time $t=0$ is not 0, but the
+#'               premium paid. In turn, no unearned premiums are applied.}
 #' }
 #'
 #' ## Elements of sublist \code{InsuranceContract.ParameterDefault$ProfitParticipation}
@@ -338,7 +476,6 @@ InsuranceContract.Values = list(
 #'               contracts (e.g. following similar risks) together. Profit
 #'               participation rates are defined at the level of profit classes.}
 #'     \item{\code{$profitRates}}{General, company-wide profit rates, key columns are year and profitClass}
-#'
 #'     \item{\code{$scenarios}}{profit participation scenarios (list of overridden parameters for each scenario)}
 #' }
 #'
@@ -347,10 +484,10 @@ InsuranceContract.Values = list(
 #' \describe{
 #'     \item{\code{$adjustCashFlows}}{Function with signature \code{function(x, params, values, ...)} to adjust the benefit/premium cash flows after their setup.}
 #'     \item{\code{$adjustCashFlowsCosts}}{Function with signature \code{function(x, params, values, ...)} to adjust the costs cash flows after their setup.}
+#'     \item{\code{$adjustPremiumCoefficients}}{Function with signature \code{function(coeff, type, premiums, params, values, premiumCalculationTime)} to adjust the coefficients for premium calculation after their default setup. Use cases are e.g. term-fix tariffs where the Zillmer premium term contains the administration cost over the whole contract, but not other gamma- or beta-costs.}
 #' }
 #'
 #'
-#'
 #' @examples
 #' InsuranceContract.ParameterDefaults
 #' @export
@@ -368,6 +505,7 @@ InsuranceContract.ParameterDefaults = list(
         deferralPeriod = 0,                     # deferral period for annuities
         guaranteedPeriod = 0,                   # guaranteed payments for annuities
         contractClosing = NULL,                 # Contract closing date (day/month is relevant for balance sheet reserves)
+        initialCapital = 0,
         blockStart = 0,                         # When the current tariff block starts (main block starts a 0, dynamic increases start later!), only used by the parent block (i.e. t=0 of child is aligned with t=blockStart of parent)
 
         premiumPayments = "in advance",         # premium payments in advance or arrears
@@ -381,7 +519,9 @@ InsuranceContract.ParameterDefaults = list(
         premiumRefund = 0,                      # Proportion of premiums refunded on death (including additional risk, e.g. 1.10 = 110% of paid premiums)
         premiumIncrease = 1,                    # The yearly growth factor of the premium, i.e. 1.05 means +5% increase each year; a Vector describes the premiums for all years
         annuityIncrease = 1,                    # The yearly growth factor of the annuity payments, i.e. 1.05 means +5% incrase each year; a vector describes the annuity unit payments for all years
-        deathBenefit = 1                        # The yearly relative death benefit (relative to the initial sum insured); Can be set to a function(len, params, values), e.g. deathBenefit = deathBenefit.linearDecreasing
+        deathBenefit = 1,                        # The yearly relative death benefit (relative to the initial sum insured); Can be set to a function(len, params, values), e.g. deathBenefit = deathBenefit.linearDecreasing
+
+        costWaiver = 0                          # The cost waiver (up to minCosts, 0=no cost waiver, 1=full cost waiver down to minCosts)
     ),
     ContractState = list(
         premiumWaiver = FALSE,                  # contract is paid-up
@@ -395,12 +535,15 @@ InsuranceContract.ParameterDefaults = list(
         i = 0.00,                               # guaranteed interest rate
         balanceSheetDate = as.Date("1900-12-31"),  # Balance sheet date (for the calculation of the balance sheet reserves, year is irrelevant)
         balanceSheetMethod = "30/360",
+        unearnedPremiumsMethod = NULL,          # Function to calculate the factors for unearned premiums
         surrenderValueCalculation = NULL,       # By default no surrender penalties
+        premiumWaiverValueCalculation = NULL,   # By default, surrender value will be used
 
         premiumFrequencyOrder = 0,              # Order of the approximation for payments within the year (unless an extra frequency loading is used => then leave this at 0)
         benefitFrequencyOrder = 0
     ),
     Costs = initializeCosts(),
+    minCosts = NULL,               # Base costs, which cannot be waived
     Loadings = list( # Loadings can also be function(sumInsured, premiums)
         ongoingAlphaGrossPremium = 0,           # Acquisition cost that increase the gross premium
         tax = 0.04,                             # insurance tax, factor on each premium paid
@@ -417,9 +560,10 @@ InsuranceContract.ParameterDefaults = list(
         premiumFrequencyLoading = list("1" = 0.0, "2" = 0.0, "4" = 0.0, "12" = 0.0), # TODO: Properly implement this as a function
         alphaRefundPeriod = 5                   # How long acquisition costs should be refunded in case of surrender
     ),
-    Features = list(                          # Special cases for the calculations
+    Features = list(                            # Special cases for the calculations
         betaGammaInZillmer = FALSE,             # Whether beta and gamma-costs should be included in the Zillmer premium calculation
-        alphaRefundLinear  = TRUE               # Whether the refund of alpha-costs on surrender is linear in t or follows the NPV of an annuity
+        alphaRefundLinear  = TRUE,              # Whether the refund of alpha-costs on surrender is linear in t or follows the NPV of an annuity
+        useUnearnedPremiums = isRegularPremiumContract # Whether unearned premiums should be calculated in the balance sheet reserves. Otherwise, a premium paid at the beginning of the period is added to the reserve for balance-sheet purposes.
     ),
 
     ProfitParticipation = list(
@@ -437,7 +581,7 @@ InsuranceContract.ParameterDefaults = list(
         terminalBonusRate = NULL,
         terminalBonusFundRate = NULL,
 
-        profitParticipationScheme = NULL,      # Gewinnbeteiligungssystem (object of class Profit Participation)
+        profitParticipationScheme = NULL,       # Gewinnbeteiligungssystem (object of class Profit Participation)
         profitComponents = c(),                 # Potential values: "interest", "risk", "expense", "sum", "terminal", "TBF"
         profitClass = NULL,
         profitRates = NULL,                     # General, company-wide profit rates, key columns are year and profitClass
@@ -448,7 +592,8 @@ InsuranceContract.ParameterDefaults = list(
     Hooks = list(
       # Functions with signature function(x, params, values, ...), default NULL is equivalent to function(x, ...) {x}
       adjustCashFlows = NULL,
-      adjustCashFlowsCosts = NULL
+      adjustCashFlowsCosts = NULL,
+      adjustPremiumCoefficients = NULL # function(coeff, type = type, premiums = premiums, params = params, values = values, premiumCalculationTime = premiumCalculationTime)
     )
 );
 
@@ -473,12 +618,12 @@ InsuranceContract.ParameterStructure$Loadings["premiumFrequencyLoading"] = list(
 #' structure is given, an empty (i.e. all NULL entries) structure is used.
 #'
 #' @param params Initial values of the insurance contract parameters. (default: empty parameter structure)
-#' @param costs,... Values for any of the entries in the insurance contract
+#' @param costs,minCosts,... Values for any of the entries in the insurance contract
 #'                  parameter structure. These values take precedence over the
 #'                  initial parameters provided in \code{params}.
 #'
 #' @export
-InsuranceContract.ParametersFill = function(params = InsuranceContract.ParameterStructure, costs = NULL, ...) {
+InsuranceContract.ParametersFill = function(params = InsuranceContract.ParameterStructure, costs = NULL, minCosts = NULL, ...) {
     # params = InsuranceContract.ParameterStructure;
     params$ContractData = fillFields(params$ContractData, list(...));
     params$ContractState = fillFields(params$ContractState, list(...));
@@ -491,6 +636,7 @@ InsuranceContract.ParametersFill = function(params = InsuranceContract.Parameter
     # Costs are a special case, because they are an array rather than a list:
     # TODO: Find a way to partially override
     if (!missing(costs)) params$Costs = costs;
+    if (!missing(minCosts)) params$minCosts = minCosts;
     params
 }
 
@@ -529,6 +675,9 @@ InsuranceContract.ParametersFallback = function(params, fallback, ppParameters =
             params$Costs = fallback$Costs;
         }
     }
+    if (is.null(params$minCosts)) {
+      params$minCosts = fallback$minCosts;
+    }
     params
 }
 
